//===========================================================================
//
// File: periodic_test.cpp
//
// Created: Fri Aug 14 13:03:17 2009
//
// Author(s): Atgeirr F Rasmussen <atgeirr@sintef.no>
//            Brd Skaflestad     <bard.skaflestad@sintef.no>
//
// $Date$
//
// $Revision$
//
//===========================================================================

/*
  Copyright 2009, 2010 SINTEF ICT, Applied Mathematics.
  Copyright 2009, 2010 Statoil ASA.

  This file is part of The Open Reservoir Simulator Project (OpenRS).

  OpenRS is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  OpenRS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with OpenRS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <config.h>

#include <opm/core/utility/have_boost_redef.hpp>

#include "../PeriodicHelpers.hpp"
#include <opm/porsol/common/BoundaryConditions.hpp>
#include <opm/porsol/common/GridInterfaceEuler.hpp>
#include <dune/grid/CpGrid.hpp>
#include <dune/common/array.hh>
#include <opm/core/utility/parameters/ParameterGroup.hpp>

using namespace Opm;

int main(int argc, char** argv)
{
    Opm::parameter::ParameterGroup param(argc, argv);
    Dune::CpGrid grid;
    grid.init(param);
    grid.setUniqueBoundaryIds(true);
    GridInterfaceEuler<Dune::CpGrid> gi(grid);
    typedef FlowBC FBC;
    Dune::array<FBC, 6> fcond = {{ FBC(FBC::Periodic, 1.0e5),
                                   FBC(FBC::Periodic, -1.0e5),
                                   FBC(FBC::Periodic, 0.0),
                                   FBC(FBC::Periodic, 0.0),
                                   FBC(FBC::Neumann, 0.0),
                                   FBC(FBC::Neumann, 0.0) }};
    typedef SatBC SBC;
    Dune::array<SBC, 6> scond = {{ SBC(SBC::Periodic, 0.0),
                                   SBC(SBC::Periodic, 0.0),
                                   SBC(SBC::Periodic, 0.0),
                                   SBC(SBC::Periodic, 0.0),
                                   SBC(SBC::Dirichlet, 0.0),
                                   SBC(SBC::Dirichlet, 0.0) }};
    BasicBoundaryConditions<true, true> bcs;
    createPeriodic(bcs, gi, fcond, scond);
    std::cout << bcs;
}

